@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtQuick module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qquickview.h"
 #include "qquickview_p.h"
@@ -49,6 +13,8 @@
 #include <private/qv4qobjectwrapper_p.h>
 #include <QtCore/qbasictimer.h>
 
+#include <memory>
+
 QT_BEGIN_NAMESPACE
 
 void QQuickViewPrivate::init(QQmlEngine* e)
@@ -74,7 +40,7 @@ void QQuickViewPrivate::init(QQmlEngine* e)
 }
 
 QQuickViewPrivate::QQuickViewPrivate()
-    : root(nullptr), component(nullptr), resizeMode(QQuickView::SizeViewToRootObject), initialSize(0,0)
+    : component(nullptr), resizeMode(QQuickView::SizeViewToRootObject), initialSize(0,0)
 {
 }
 
@@ -90,10 +56,8 @@ void QQuickViewPrivate::execute()
         return;
     }
 
-    if (root) {
+    if (root)
         delete root;
-        root = nullptr;
-    }
     if (component) {
         delete component;
         component = nullptr;
@@ -209,7 +173,6 @@ QQuickView::~QQuickView()
     // be a child of the QQuickViewPrivate, and will be destroyed by its dtor
     Q_D(QQuickView);
     delete d->root;
-    d->root = nullptr;
 }
 
 /*!
@@ -243,7 +206,11 @@ void QQuickView::setSource(const QUrl& url)
    Sets the initial properties \a initialProperties with which the QML
    component gets initialized after calling \l QQuickView::setSource().
 
+   \snippet qquickview-ex.cpp 1
+
    \note You can only use this function to initialize top-level properties.
+   \note This function should always be called before setSource, as it has
+   no effect once the component has become \c Ready.
 
    \sa QQmlComponent::createWithInitialProperties()
    \since 5.14
@@ -275,7 +242,8 @@ void QQuickView::setContent(const QUrl& url, QQmlComponent *component, QObject*
         return;
     }
 
-    d->setRootObject(item);
+    if (!d->setRootObject(item))
+        delete item;
     emit statusChanged(status());
 }
 
@@ -486,9 +454,9 @@ void QQuickView::continueExecute()
         return;
     }
 
-    QScopedPointer<QObject> obj(d->initialProperties.empty()
-                                ? d->component->create()
-                                : d->component->createWithInitialProperties(d->initialProperties));
+    std::unique_ptr<QObject> obj(d->initialProperties.empty()
+                                 ? d->component->create()
+                                 : d->component->createWithInitialProperties(d->initialProperties));
 
     if (d->component->isError()) {
         const QList<QQmlError> errorList = d->component->errors();
@@ -500,43 +468,56 @@ void QQuickView::continueExecute()
         return;
     }
 
-    d->setRootObject(obj.take());
+    if (d->setRootObject(obj.get()))
+        Q_UNUSED(obj.release());
     emit statusChanged(status());
 }
 
 
 /*!
   \internal
+
+  Sets \a obj as root object and returns true if that operation succeeds.
+  Otherwise returns \c false. If \c false is returned, the root object is
+  \c nullptr afterwards. You can explicitly set the root object to nullptr,
+  and the return value will be \c true.
 */
-void QQuickViewPrivate::setRootObject(QObject *obj)
+bool QQuickViewPrivate::setRootObject(QObject *obj)
 {
     Q_Q(QQuickView);
     if (root == obj)
-        return;
+        return true;
+
+    delete root;
+    if (obj == nullptr)
+        return true;
+
     if (QQuickItem *sgItem = qobject_cast<QQuickItem *>(obj)) {
         root = sgItem;
+        root->setFlag(QQuickItem::ItemIsViewport);
         sgItem->setParentItem(q->QQuickWindow::contentItem());
         QQml_setParent_noEvent(sgItem, q->QQuickWindow::contentItem());
-    } else if (qobject_cast<QWindow *>(obj)) {
-        qWarning() << "QQuickView does not support using windows as a root item." << Qt::endl
-                   << Qt::endl
-                   << "If you wish to create your root window from QML, consider using QQmlApplicationEngine instead." << Qt::endl;
-    } else {
-        qWarning() << "QQuickView only supports loading of root objects that derive from QQuickItem." << Qt::endl
-                   << Qt::endl
-                   << "Ensure your QML code is written for QtQuick 2, and uses a root that is or" << Qt::endl
-                   << "inherits from QtQuick's Item (not a Timer, QtObject, etc)." << Qt::endl;
-        delete obj;
-        root = nullptr;
-    }
-    if (root) {
         initialSize = rootObjectSize();
         if ((resizeMode == QQuickView::SizeViewToRootObject || q->width() <= 1 || q->height() <= 1) &&
             initialSize != q->size()) {
             q->resize(initialSize);
         }
         initResize();
+        return true;
     }
+
+    if (qobject_cast<QWindow *>(obj)) {
+        qWarning() << "QQuickView does not support using a window as a root item." << Qt::endl
+                   << Qt::endl
+                   << "If you wish to create your root window from QML, consider using QQmlApplicationEngine instead." << Qt::endl;
+        return false;
+    }
+
+    qWarning() << "QQuickView only supports loading of root objects that derive from QQuickItem." << Qt::endl
+               << Qt::endl
+               << "Ensure your QML code is written for QtQuick 2, and uses a root that is or" << Qt::endl
+               << "inherits from QtQuick's Item (not a Timer, QtObject, etc)." << Qt::endl;
+    return false;
 }
 
 /*!
